#
# Copyright (C) 2007 - 2008 M <optimizationkit@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# OptimizationKit - control groups library

import os, commands
from ok_srv import write_to_file

debug = 0

def get_pids(arg, pgrep_args):
	if (debug):
		print "\nget_pids:\narg = %s\npgrep_args = %s" % (arg, pgrep_args)

	pids = []
	pids_tmp = commands.getoutput("pgrep " + pgrep_args + " " + arg)
	pids_tmp = pids_tmp.split('\n')

	for pid in pids_tmp:
		pids.append(pid)

	if (debug):
		print "\npids = %s" % pids

	return pids

def add_tasks_to_cgroup(cgroup, args, pgrep_args, cgroup_mount_dir):
	for arg in args:
		pids = get_pids(arg, pgrep_args)
		if len(pids[0]) > 0:
			for pid in pids:
				write_to_file(cgroup_mount_dir + "/" + cgroup + "/tasks", pid)

def change_default_cgroup_opts(cgroup, opts, cgroup_mount_dir):
	if (debug):
		print "\nchange_default_cgroup_opts:\ncgroup = %s\nopts = %s\ncgroup_mount_dir = %s" % (cgroup, opts, cgroup_mount_dir)

	if (debug):
		print "\ncpuset.cpu_exclusive = %s" % opts["cpuset.cpu_exclusive"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.cpu_exclusive", opts["cpuset.cpu_exclusive"])
	if (debug):
		print "\ncpuset.cpus = %s" % opts["cpuset.cpus"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.cpus", opts["cpuset.cpus"])
	if (debug):
		print "\ncpuset.mem_exclusive = %s" % opts["cpuset.mem_exclusive"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.mem_exclusive", opts["cpuset.mem_exclusive"])
	if (debug):
		print "\ncpuset.memory_migrate = %s" % opts["cpuset.memory_migrate"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.memory_migrate", opts["cpuset.memory_migrate"])
#	if (debug):
#		print "\ncpuset.memory_pressure = %s" % opts["cpuset.memory_pressure"]
#	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.memory_pressure", opts["cpuset.memory_pressure"])
	if (debug):
		print "\ncpuset.memory_spread_page = %s" % opts["cpuset.memory_spread_page"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.memory_spread_page", opts["cpuset.memory_spread_page"])
	if (debug):
		print "\ncpuset.memory_spread_slab = %s" % opts["cpuset.memory_spread_slab"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.memory_spread_slab", opts["cpuset.memory_spread_slab"])
	if (debug):
		print "\ncpuset.mems = %s" % opts["cpuset.mems"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.mems", opts["cpuset.mems"])
	if (debug):
		print "\ncpuset.sched_load_balance = %s" % opts["cpuset.sched_load_balance"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cpuset.sched_load_balance", opts["cpuset.sched_load_balance"])
	if (debug):
		print "\nnotify_on_release = %s" % opts["notify_on_release"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/notify_on_release", opts["notify_on_release"])
	if (debug):
		print "\nmemory.limit_in_bytes = %s" % opts["memory.limit_in_bytes"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/memory.limit_in_bytes", opts["memory.limit_in_bytes"])
	if (debug):
		print "\ncfq.ioprio = %s" % opts["cfq.ioprio"]
	write_to_file(cgroup_mount_dir + "/" + cgroup + "/cfq.ioprio", opts["cfq.ioprio"])

def create_cgroup(cgroup, cgroup_mount_dir):
	if (debug):
		print "\ncreate_cgroup\ncgroup = %s" % cgroup
	if os.path.isdir(cgroup_mount_dir + "/" + cgroup):
		os.system("rmdir --ignore-fail-on-non-empty " + cgroup_mount_dir + "/" + cgroup)

	os.system("mkdir " + cgroup_mount_dir + "/" + cgroup)

def create_groups_from_config(config, cgroup_mount_dir):
	if (debug):
		print "\ncreate_groups_from_config"
	for i in config:
		if (debug):
			print "\ngroup name = %s\ngroup opts = %s" % (i[0], i[6])
		if i[6] != "NULL":
			create_cgroup(i[0], cgroup_mount_dir)
			change_default_cgroup_opts(i[0], i[6], cgroup_mount_dir)

def setup_cgroups(cpu_exclusive, mem_exclusive, features, cgroup_mount_dir):
	mount_opts = ""
	if (debug):
		print "\nsetup_cgroups features = %s\n" % features
	if os.path.isdir(cgroup_mount_dir):
		os.system("umount " + cgroup_mount_dir)
	else:
		os.system("mkdir " + cgroup_mount_dir)

	if features["has_cpuset"] == 1:
		mount_opts += "cpuset"

	if (features["has_memory"] == 1) and (len(mount_opts) > 0):
		mount_opts += ",memory"
	elif (features["has_memory"] == 1) and (len(mount_opts) == 0):
		mount_opts += "memory"

	if (features["has_cfqcg"] == 1) and (len(mount_opts) > 0):
		mount_opts += ",cfq"
	elif (features["has_cfqcg"] == 1) and (len(mount_opts) == 0):
		mount_opts += "cfq"

	os.system("mount -t cgroup none -o " + mount_opts + " " + cgroup_mount_dir)
	write_to_file(cgroup_mount_dir + "/cpuset.cpu_exclusive", cpu_exclusive)
	write_to_file(cgroup_mount_dir + "/cpuset.mem_exclusive", mem_exclusive)